from markdown.extensions import Extension
from markdown.inlinepatterns import SimpleTagInlineProcessor

from jiramd.code import code_processor
from jiramd.link import link_processor
from jiramd.monospace import monospace_processor

STRONG_RE = r'(?<!\w)(\*)([^\*]+)\1(?!\w)'
EMPHASIS_RE = r'(?<!\w)(_)(.+?)\1(?!\w)'


class JiraWikiExtension(Extension):
    def extendMarkdown(self, md):
        md.inlinePatterns.register(monospace_processor, 'jira-monospace', 200)
        md.inlinePatterns.register(code_processor, 'jira-code', 205)
        md.inlinePatterns.register(link_processor, 'jira-link', 210)
        md.inlinePatterns.register(SimpleTagInlineProcessor(STRONG_RE, 'strong'), 'jira-strong', 199)
        md.inlinePatterns.register(SimpleTagInlineProcessor(EMPHASIS_RE, 'em'), 'jira-emphasis', 198)
        md.inlinePatterns.deregister('strong')
        md.inlinePatterns.deregister('strong2')
        md.inlinePatterns.deregister('emphasis')
        md.inlinePatterns.deregister('emphasis2')



from markdown.inlinepatterns import InlineProcessor
from markdown.util import etree, code_escape, AtomicString

CODE_PATTERN = r'(\{code\})(.+?)\1'


class CodeProcessor(InlineProcessor):
    def handleMatch(self, m, data):
        pre = etree.Element('pre')
        code = etree.SubElement(pre, 'code')
        code.text = AtomicString(code_escape(m.group(2)))
        return pre, m.start(0), m.end(0)


code_processor = CodeProcessor(CODE_PATTERN)



from markdown.inlinepatterns import InlineProcessor
from markdown.util import etree

LINK_PATTERN = r'\[([^|]+)\|https?://(.*?)\]'


class LinkProcessor(InlineProcessor):
    def handleMatch(self, m, data):
        a = etree.Element('a')
        print(m.groups())
        a.set('href', 'https://' + m.group(2))
        a.text = m.group(1)
        return a, m.start(0), m.end(0)


link_processor = LinkProcessor(LINK_PATTERN)



import markdown

from jiramd.extension import JiraWikiExtension


def test_monospace():
    text = "foo {{bar}} quz"

    md = markdown.markdown(text, extensions=[JiraWikiExtension()])

    assert md == "<p>foo <code>bar</code> quz</p>"


def test_monospace_with_html():
    text = "foo {{bar<br>}}"

    md = markdown.markdown(text, extensions=[JiraWikiExtension()])

    assert md == "<p>foo <code>bar&lt;br&gt;</code></p>"


def test_monospace_with_markdown():
    text = "foo {{*bar*}}"

    md = markdown.markdown(text, extensions=[JiraWikiExtension()])

    assert md == "<p>foo <code>*bar*</code></p>"


def test_code():
    text = "foo\n{code}\nbar\n{code}"

    md = markdown.markdown(text, extensions=[JiraWikiExtension()])

    assert md == "<p>foo\n<pre><code>\nbar\n</code></pre>\n</p>"


def test_code_with_html():
    text = "foo\n{code}\nbar<br>\n{code}"

    md = markdown.markdown(text, extensions=[JiraWikiExtension()])

    assert md == "<p>foo\n<pre><code>\nbar&lt;br&gt;\n</code></pre>\n</p>"


def test_code_with_markdown():
    text = "foo\n{code}\n*bar*\n{code}"

    md = markdown.markdown(text, extensions=[JiraWikiExtension()])

    assert md == "<p>foo\n<pre><code>\n*bar*\n</code></pre>\n</p>"


def test_jira_strong():
    text = '*foo*'

    md = markdown.markdown(text, extensions=[JiraWikiExtension()])

    assert md == '<p><strong>foo</strong></p>'


def test_jira_emphasis():
    text = '_foo_'

    md = markdown.markdown(text, extensions=[JiraWikiExtension()])

    assert md == '<p><em>foo</em></p>'


def test_jira_emphasis_unspaced():
    text = 'this_foo_bar'

    md = markdown.markdown(text, extensions=[JiraWikiExtension()])

    assert md == '<p>this_foo_bar</p>'


def test_jira_emphasis_end_sentence():
    text = 'this is _foo_.'

    md = markdown.markdown(text, extensions=[JiraWikiExtension()])

    assert md == '<p>this is <em>foo</em>.</p>'


def test_link():
    text = '[foo|https://test.org/]'

    md = markdown.markdown(text, extensions=[JiraWikiExtension()])

    assert md == '<p><a href="https://test.org/">foo</a></p>'



from markdown.inlinepatterns import InlineProcessor
from markdown.util import etree, code_escape, AtomicString

MONOSPACE_PATTERN = r'\{\{(.+?)\}\}'


class MonospaceProcessor(InlineProcessor):
    def handleMatch(self, m, data):
        pre = etree.Element('code')
        pre.text = AtomicString(code_escape(m.group(1)))
        return pre, m.start(0), m.end(0)


monospace_processor = MonospaceProcessor(MONOSPACE_PATTERN)



from setuptools import setup, find_packages

setup(name='jira-wiki-md',
      version='0.4.2',
      author='Jean Giard',
      license='LGPL',
      classifier=[
          'Programming Language :: Python :: 3'
      ],
      packages=find_packages(where='src'),
      package_dir={'': 'src'},
      entry_points={
          'markdown.extensions': ['jirawiki = jiramd.extension:JiraWikiExtension']
      },
      include_package_data=True,
      install_requires=[
          'markdown'
      ],
      )
